import os
import re
import urllib3

from urllib import parse

from .. import BaseService

from appd.request import AppDynamicsClient


class Service(BaseService):
    _conn_params = {}

    application = None

    def __init__(self, scheme, conn_str):
        super(Service, self).__init__(scheme, conn_str)

        pattern = re.compile(
            r"""
                (?P<username>[^:/]*)
                :(?P<password>.*)
                @(?:
                    \[(?P<ipv6host>[^/]+)\] |
                    (?P<ipv4host>[^/:]+)
                )
                (?::(?P<port>[\d]+))?
                (?:/(?P<application>.*))?
            """,
            re.X
        )

        m = pattern.match(self._conn_str)

        if m is not None:
            components = m.groupdict()

            components['username'] = parse.unquote(components['username'])
            components['password'] = parse.unquote(components['password'])

            ipv4host = components.pop('ipv4host')
            ipv6host = components.pop('ipv6host')

            host = ipv4host or ipv6host

            if self._scheme is not None:
                scheme = self._scheme
            else:
                scheme = 'https'

            components['base_url'] = scheme + '://' + host

            port = components.pop('port')

            if port is not None:
                components['base_url'] += ':' + port

            application = components.pop('application')

            if application is not None:
                self.application = parse.unquote(application)

            self._conn_params = components

    @property
    def client(self):
        verify = os.getenv('APIALCHEMY_APPD_SSL_VERIFY', 'true').lower() == 'true'

        if not verify:
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

        return AppDynamicsClient(**self._conn_params, verify=verify)

    @staticmethod
    def get_application_component_id(client, application_id, name):
        result = client.get_tiers(application_id)

        for t in result:
            if t.name == name:
                return t.id

        return None

    @staticmethod
    def get_application_id(client, name):
        for a in client.get_applications():
            if a.name == name:
                return a.id

        return None

    @staticmethod
    def get_business_transaction_id(client, application_id, name):
        result = client.get_bt_list(application_id)

        for bt in result:
            if bt.name == name:
                return bt.id

        return None



import base64
import ssl

from prometheus_client import delete_from_gateway, push_to_gateway, pushadd_to_gateway

from urllib.request import build_opener, Request, HTTPHandler, HTTPSHandler
from urllib.parse import urlparse

DEFAULT_HOST = 'localhost'
DEFAULT_SCHEME = 'https'


class PushgatewayClient:
    _session = None

    def __init__(self, **kwargs):
        self.base_url = self._get_base_url(kwargs.get('scheme', DEFAULT_SCHEME),
                                           kwargs.get('host', DEFAULT_HOST),
                                           kwargs.get('port'))
        self.username = kwargs.get('username')
        self.password = kwargs.get('password')
        self.verify = kwargs.get('verify', True)

    def _get_base_url(self, scheme, host, port):
        if ':' in host:
            host = '[' + host + ']'

        base_url = scheme + '://' + host

        if port is not None:
            base_url += ':' + port

        return base_url

    def _pushgateway_handler(self, url, method, timeout, headers, data):
        username = self.username
        password = self.password
        verify = self.verify

        def handle():
            if username is not None and password is not None:
                auth_value = '{0}:{1}'.format(username, password).encode('utf-8')
                auth_token = base64.b64encode(auth_value)
                auth_header = b'Basic ' + auth_token
                headers.append(['Authorization', auth_header])

            url_components = urlparse(url)

            if not url_components.scheme or url_components.scheme != 'https':
                url_handler = HTTPHandler
            else:
                context = ssl.SSLContext()

                if verify:
                    context.verify_mode = ssl.CERT_REQUIRED
                else:
                    context.verify_mode = ssl.CERT_NONE

                url_handler = HTTPSHandler(context=context)

            request = Request(url, data=data)
            request.get_method = lambda: method

            for k, v in headers:
                request.add_header(k, v)

            resp = build_opener(url_handler).open(request, timeout=timeout)

            if resp.code >= 400:
                raise IOError("error talking to pushgateway: {0} {1}".format(
                    resp.code, resp.msg))

        return handle

    def delete(self, **kwargs):
        if 'handler' in kwargs:
            handler = kwargs.pop('handler')
        else:
            handler = None

        if not callable(handler):
            handler = self._pushgateway_handler

        delete_from_gateway(self.base_url, handler=handler, **kwargs)

    def push(self, **kwargs):
        if 'handler' in kwargs:
            handler = kwargs.pop('handler')
        else:
            handler = None

        if not callable(handler):
            handler = self._pushgateway_handler

        strict = kwargs.pop('strict') if 'strict' in kwargs.keys() else False

        if strict:
            pushadd_to_gateway(self.base_url, handler=handler, **kwargs)
        else:
            push_to_gateway(self.base_url, handler=handler, **kwargs)



import os
import re
import urllib3

from github import Github

from urllib import parse

from .. import BaseService


class Service(BaseService):
    _conn_params = {}

    def __init__(self, scheme, conn_str):
        super(Service, self).__init__(scheme, conn_str)

        pattern = re.compile(
            r"""
                (?P<login_or_token>.*)
                @(?:
                    \[(?P<ipv6host>[^/]+)\] |
                    (?P<ipv4host>[^/:]+)
                )
                (?::(?P<port>[\d]+))?
            """,
            re.X
        )

        m = pattern.match(self._conn_str)

        if m is not None:
            components = m.groupdict()

            components['login_or_token'] = parse.unquote(components['login_or_token'])

            ipv4host = components.pop('ipv4host')
            ipv6host = components.pop('ipv6host')

            host = ipv4host or ipv6host

            if self._scheme is not None:
                scheme = self._scheme
            else:
                scheme = 'https'

            components['base_url'] = scheme + '://' + host

            port = components.pop('port')

            if port is not None:
                components['base_url'] += ':' + port

            components['base_url'] += '/api/v3'

            self._conn_params = components

    @property
    def client(self):
        verify = os.getenv('APIALCHEMY_GITHUB_SSL_VERIFY', 'true').lower() == 'true'

        if not verify:
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

        return Github(**self._conn_params, verify=verify)

    @staticmethod
    def get_file_contents(client, org, repo, path, branch):
        repo = client.get_repo(org + '/' + repo)

        contents = repo.get_contents(path, ref=branch)

        return contents.decoded_content



import json
import sys

from requests.sessions import Session
from requests import Request, codes

DEFAULT_HOST = 'localhost'
DEFAULT_SCHEME = 'https'


class ExtraHopClient:
    _session = None

    def __init__(self, **kwargs):
        self.base_url = self._get_base_url(kwargs.get('scheme', DEFAULT_SCHEME),
                                           kwargs.get('host', DEFAULT_HOST),
                                           kwargs.get('port'))
        self.apikey = kwargs.get('apikey')
        self.verify = kwargs.get('verify', True)

    def _get_base_url(self, scheme, host, port):
        if ':' in host:
            host = '[' + host + ']'

        base_url = scheme + '://' + host

        if port is not None:
            base_url += ':' + port

        return base_url

    def _get_full_path(self, path=None):
        path = '/api/v1' + (path or '')

        return path

    def _get_session(self):
        if not self._session:
            self._session = Session()
            self._session.verify = self.verify

        return self._session

    def _request(self, **request_args):
        s = self._get_session()

        req = Request(**request_args)

        prepped = s.prepare_request(req)

        # Merge environment settings into session
        settings = s.merge_environment_settings(prepped.url, {}, None, None, None)

        return s.send(prepped, **settings)

    def get_metrics(self, **kwargs):
        path = self._get_full_path('/metrics')

        return self.request(path, kwargs, method='POST', query=False)

    def request(self, path, params=None, method='GET', use_json=True, query=True, headers=None):
        if not path.startswith('/'):
            path = '/' + path

        url = self.base_url + path

        params = params or {}

        for k in list(params.keys()):
            if params[k] is None:
                del params[k]

        headers = headers or {}

        kwargs = {
            'method': method,
            'url': url,
            'headers': {
                'Authorization': 'ExtraHop apikey=' + self.apikey
            }
        }

        if method == 'GET' or query:
            kwargs['params'] = params
        else:
            kwargs['data'] = json.dumps(params)

            if not headers:
                headers = {
                    'Content-type': 'application/json',
                    'Accept': 'text/plain'
                }

        kwargs['headers'].update(headers)

        r = self._request(**kwargs)

        if r.status_code != codes.ok:
            print(url, file=sys.stderr)

            r.raise_for_status()

        return r.json() if use_json else r.text
